import java.util.*;
import java.io.*;
import java.security.*;
public class Runner{
    double[] seq;
    double[] marked;
    int L = 10000;
    int N, C;
    Random r;
    Process proc;
    Scanner sc;
    OutputStream os;

    public static void main(String[] args) throws IOException{
        int seed = 1;
        String exec = null;
        for(int i = 0; i<args.length; i++){
            if(args[i].equals("-seed")){
                seed = Integer.parseInt(args[++i]);
            }else if(args[i].equals("-exec")){
                exec = args[++i];
            }
        }
        new Runner(seed,exec);
    }
    public double[] mark(double[] seq, int N, int C) throws IOException{
        StringBuffer sb = new StringBuffer(seq.length * 20);
        sb.append(N).append('\n');
        sb.append(C).append('\n');
        sb.append(seq.length).append('\n');
        sb.append(seq[0]);
        for(int i = 1; i<seq.length; i++){
            sb.append(' ').append(seq[i]);
        }
        sb.append('\n');
        os.write(sb.toString().getBytes());
        os.flush();
        double[] ret = new double[seq.length*N];
        for(int i = 0; i<ret.length; i++)
            ret[i] = sc.nextDouble();
        return ret;
    }
    public int identify(double[] seq) throws IOException{
        StringBuffer sb = new StringBuffer(seq.length * 20);
        sb.append(seq.length).append('\n');
        sb.append(seq[0]);
        for(int i = 1; i<seq.length; i++){
            sb.append(' ').append(seq[i]);
        }
        sb.append('\n');
        os.write(sb.toString().getBytes());
        os.flush();
        return sc.nextInt();
    }
    public Runner(int seed, String exec) throws IOException{
        Runtime rt = Runtime.getRuntime();
        proc = rt.exec(exec);
        os = proc.getOutputStream();
        sc = new Scanner(proc.getInputStream());
        new ErrorReader(proc.getErrorStream()).start();

        try{
            r = SecureRandom.getInstance("SHA1PRNG");
            r.setSeed(seed);
        }catch(Exception e){
            e.printStackTrace();
        }
        N = r.nextInt(99)+2;
        C = r.nextInt(10)+1;
        if(seed == 10) C = 10;
        if(seed >= 1 && seed <= 10)
            L = r.nextInt(9901)+1000; //this is wrong, but the images were already generated
        else
            L = r.nextInt(9001)+1000;
        generate();
        marked = mark(seq,N,C);
        if(marked.length != N * L){
            System.out.println("Your return from mark had the wrong number of elements.");
            return;
        }
        int inc = 0;
        for(int i = 0; i<100; i++){
            int id = r.nextInt(N);
            double[] d = perturb(id);
            int val = identify(d);
            if(id != val){
                inc++;
            }
        }
        System.out.println("You got "+inc+" of the identification tasks wrong.");
        double se = 0;
        for(int i = 0; i<marked.length; i++){
            double d = marked[i] - seq[i%L];
            se += d*d;
        }
        System.out.println("Score = "+(se + inc) * Math.pow(2,inc));
    }
    double[] perturb(int id){
        double[] ret = new double[L];
        double shift = r.nextGaussian() * C * 10;
        for(int i = 0 ;i<L; i++){
            ret[i] = marked[i+id*L] + shift + r.nextGaussian() * C;
        }
        double p = C/20.0;
        int skip = 0;
        for(int i = 0 ; i < L; i++){
            p += r.nextGaussian() * 0.01;
            if(p < 0)p = 0;
            else if(p > C/10.0)p = C/10.0;
            if(r.nextDouble() < p){
                skip++;
            }else{
                ret[i-skip] = ret[i];
            }
        }
        double[] rr = new double[L-skip-C+1];
        double avg = 0;
        for(int i = 0; i<rr.length + C - 1;i++){
            avg += ret[i];
            if(i >= C - 1){
                rr[i - C + 1] = avg/C;
                avg -= ret[i - C + 1];
            }
        }
        return rr;
    }
    void generate(){
        try{
            seq = new double[L];
            double d0 = 0;
            double d1 = 0;
            double d2 = 0;
            double d3 = 0;
            for(int i = 0; i<L; i++){
                d3 *= 0.99;
                d2 *= 0.99;
                d1 *= 0.99;
                d0 *= 0.99;
                d3 += r.nextGaussian()/1000;
                d2 += d3;
                d1 += d2;
                d0 += d1;
                seq[i] = d0;
            }
        }catch(Exception e){
            e.printStackTrace();
        }
    }
    public String checkData(String s){return "";}
}


class ErrorReader extends Thread{
    InputStream error;
    public ErrorReader(InputStream is) {
        error = is;
    }
    public void run() {
        try {
            byte[] ch = new byte[50000];
            int read;
            while ((read = error.read(ch)) > 0)
	    {   String s = new String(ch,0,read);
                System.out.print(s);
                System.out.flush();
            }
        } catch(Exception e) { }
    }
}
